@@ -71,7 +71,7 @@ void parse_iwpm_config(FILE *fp)
 	while (str) {
 		if (line_buf[0] == '#' || line_buf[0] == '\n')
 			goto parse_next_line;
-		n = sscanf(line_buf, "%64[^= ] %*[=]%d", param_name, &val);
+		n = sscanf(line_buf, "%63[^= ] %*[=]%d", param_name, &val);
 		if (n != 2) {
 			syslog(LOG_WARNING, "parse_iwpm_config: Couldn't parse a line (n = %d, name = %s, val = %d\n", n, param_name, val);
 			goto parse_next_line;
@@ -211,7 +211,8 @@ int create_netlink_socket(void)
 	sockaddr_union bind_addr;
 	struct sockaddr_nl *bind_nl;
 	int nl_sock;
-	__u32 rbuf_size, opt_len;
+	__u32 rbuf_size;
+	socklen_t opt_len;
 
 	/* create a socket */
 	nl_sock = socket(AF_NETLINK, SOCK_RAW, NETLINK_RDMA);
@@ -252,8 +253,16 @@ int create_netlink_socket(void)
                		}
 		}
 	}
-	getsockopt(nl_sock, SOL_SOCKET, SO_RCVBUF, &rbuf_size, &opt_len);
-	iwpm_debug(IWARP_PM_NETLINK_DBG, "create_netlink_socket: Setting a sock option (rbuf_size = %u).\n", rbuf_size);
+
+	opt_len = sizeof(rbuf_size);
+	if (getsockopt(nl_sock, SOL_SOCKET, SO_RCVBUF, &rbuf_size, &opt_len)) {
+		iwpm_debug(IWARP_PM_NETLINK_DBG,
+			   "create_netlink_socket: Setting a sock option (rbuf_size = %u).\n",
+			   rbuf_size);
+	} else {
+		syslog(LOG_WARNING, "create_netlink_socket: Failed to get socket option. %s.\n",
+			strerror(errno));
+	}
 
 create_nl_socket_exit:
 	return nl_sock;
@@ -264,7 +273,7 @@ create_nl_socket_exit:
  */
 void destroy_iwpm_socket(int pm_sock)
 {
-	if (pm_sock > 0)
+	if (pm_sock >= 0)
 		close(pm_sock);
 	pm_sock = -1;
 }
@@ -570,6 +579,10 @@ void copy_iwpm_sockaddr(__u16 addr_family, struct sockaddr_storage *src_sockaddr
 		*dst = *src;
 		break;
 	}
+	default:
+	assert(false);
+	if (dst_sockaddr)
+		dst_sockaddr->ss_family = addr_family;
 	}
 }
 
