@@ -90,6 +90,9 @@ struct guid_ca_pairs_mapping {
 	struct umad_ca_pair *ca_pair;
 };
 
+typedef struct { char _ca_names_arr[UMAD_MAX_DEVICES][UMAD_CA_NAME_LEN]; } ca_names_t;
+
+
 #define IBWARN(fmt, args...) fprintf(stderr, "ibwarn: [%d] %s: " fmt "\n", getpid(), __func__, ## args)
 
 #define TRACE	if (umaddebug)	IBWARN
@@ -1476,24 +1479,28 @@ static int count_ports_by_guid(char legacy_ca_names[][UMAD_CA_NAME_LEN], size_t
 int umad_get_smi_gsi_pairs(struct umad_ca_pair cas[], size_t max)
 {
 	size_t added_devices = 0, added_mappings = 0;
-	char legacy_ca_names[UMAD_MAX_DEVICES][UMAD_CA_NAME_LEN] = {};
+	ca_names_t *legacy_ca_names = malloc(sizeof(ca_names_t));
+	if (!legacy_ca_names)
+		return -1;
 	struct port_guid_port_count counts[UMAD_MAX_PORTS] = {};
 	struct guid_ca_pairs_mapping mapping[UMAD_MAX_PORTS] = {};
 
 	memset(cas, 0, sizeof(struct umad_ca_pair) * max);
-	int cas_found = umad_get_cas_names(legacy_ca_names, UMAD_MAX_DEVICES);
+	int cas_found = umad_get_cas_names(legacy_ca_names->_ca_names_arr, UMAD_MAX_DEVICES);
 
-	if (cas_found < 0)
+	if (cas_found < 0) {
+		free(legacy_ca_names);
 		return 0;
+	}
 
-	count_ports_by_guid(legacy_ca_names, cas_found, counts, UMAD_MAX_PORTS);
+	count_ports_by_guid(legacy_ca_names->_ca_names_arr, cas_found, counts, UMAD_MAX_PORTS);
 
 	size_t c_idx = 0;
 
 	for (c_idx = 0; c_idx < (size_t)cas_found; ++c_idx) {
 		umad_ca_t curr_ca;
 
-		if (umad_get_ca(legacy_ca_names[c_idx], &curr_ca) < 0)
+		if (umad_get_ca(legacy_ca_names->_ca_names_arr[c_idx], &curr_ca) < 0)
 			continue;
 
 		size_t p_idx = 0;
@@ -1525,6 +1532,7 @@ int umad_get_smi_gsi_pairs(struct umad_ca_pair cas[], size_t max)
 				break;
 			} else {
 				umad_release_ca(&curr_ca);
+				free(legacy_ca_names);
 				return -1;
 			}
 		}
@@ -1532,6 +1540,7 @@ int umad_get_smi_gsi_pairs(struct umad_ca_pair cas[], size_t max)
 		umad_release_ca(&curr_ca);
 	}
 
+	free(legacy_ca_names);
 	return added_devices;
 }
 
@@ -1568,12 +1577,22 @@ static int umad_find_active(struct umad_ca_pair *ca_pair, const umad_ca_t *ca, b
 	return 1;
 }
 
-static int find_preferred_ports(struct umad_ca_pair *ca_pair, const umad_ca_t *ca, bool is_gsi, int portnum)
+static int find_preferred_ports(struct umad_ca_pair *ca_pair, const umad_ca_t *ca, bool is_gsi, int portnum, const char *name)
 {
-	if (portnum) {
+	if (ca && ca->numports == 1) {
+		size_t i = 0;
+		for (i = 0; i < (size_t)ca->numports + 1; ++i) {
+			if (ca->ports[i]) {
+				portnum = ca->ports[i]->portnum;
+				break;
+			}
+		}
+	}
+	if (portnum || (portnum == 0 && ca && ca->node_type == 2)) {
 		//in case we have same device, use same port for smi/gsi
 		if (!strncmp(ca_pair->gsi_name, ca_pair->smi_name, UMAD_CA_NAME_LEN)) {
-			if (!umad_check_active(ca, portnum)) {
+			//in case of explicit portnum and device name, no need to check active device
+			if ((name) || !umad_check_active(ca, portnum)) {
 				ca_pair->gsi_preferred_port = portnum;
 				ca_pair->smi_preferred_port = portnum;
 				return 0;
@@ -1583,9 +1602,11 @@ static int find_preferred_ports(struct umad_ca_pair *ca_pair, const umad_ca_t *c
 		uint32_t *port_to_set = is_gsi ?
 					&ca_pair->gsi_preferred_port :
 					&ca_pair->smi_preferred_port;
-		if (!umad_check_active(ca, portnum)) {
+		if ((name) || !umad_check_active(ca, portnum)) {
 			*port_to_set = portnum;
-			return umad_find_active(ca_pair, ca, !is_gsi);
+			//if find active device falis, do not fail if user chose explicit device name and portnum
+			int rc = umad_find_active(ca_pair, ca, !is_gsi);
+			return name ? 0 : rc;
 		}
 		return 1;
 	}
@@ -1648,7 +1669,7 @@ int umad_get_smi_gsi_pair_by_ca_name(const char *name, uint8_t portnum, struct u
 
 		// fill candidate
 		*ca_pair = cas_pair[i];
-		rc = find_preferred_ports(ca_pair, &ca, is_gsi, portnum);
+		rc = find_preferred_ports(ca_pair, &ca, is_gsi, portnum, name);
 		umad_release_ca(&ca);
 
 		if (!rc)
