@@ -62,7 +62,7 @@ union umad_gid {
 	} global;
 } __attribute__((aligned(4))) __attribute__((packed));
 
-#define UMAD_MAX_DEVICES 32
+#define UMAD_MAX_DEVICES 128
 #define UMAD_ANY_PORT	0
 typedef struct ib_mad_addr {
 	__be32 qpn;
@@ -169,6 +169,14 @@ typedef struct umad_ca {
 	umad_port_t *ports[UMAD_CA_MAX_PORTS];
 } umad_ca_t;
 
+struct umad_ca_pair {
+	char smi_name[UMAD_CA_NAME_LEN];
+	uint32_t smi_preferred_port;
+	char gsi_name[UMAD_CA_NAME_LEN];
+	uint32_t gsi_preferred_port;
+};
+
+
 struct umad_device_node {
 	struct umad_device_node *next; /* next umad device node  */
 	const char *ca_name; /* ca name */
@@ -188,6 +196,7 @@ int umad_release_port(umad_port_t * port);
 int umad_get_issm_path(const char *ca_name, int portnum, char path[], int max);
 
 int umad_open_port(const char *ca_name, int portnum);
+int umad_open_smi_port(const char *ca_name, int portnum);
 int umad_close_port(int portid);
 
 void *umad_get_mad(void *umad);
@@ -236,6 +245,9 @@ int umad_register2(int port_fd, struct umad_reg_attr *attr,
 int umad_debug(int level);
 void umad_addr_dump(ib_mad_addr_t * addr);
 void umad_dump(void *umad);
+int umad_get_smi_gsi_pairs(struct umad_ca_pair cas[], size_t max);
+int umad_get_smi_gsi_pair_by_ca_name(const char *devname, uint8_t portnum,
+									struct umad_ca_pair *ca_pair, unsigned enforce_smi);
 
 static inline void *umad_alloc(int num, size_t size)
 {				/* alloc array of umad buffers */
