@@ -180,7 +180,7 @@ struct mlx5_crypto_bsf {
 	__be32 raw_data_size;
 	uint8_t bs_pointer;
 	uint8_t rsvd1[7];
-	__be32 xts_init_tweak[4];
+	uint8_t xts_init_tweak[16];
 	__be32 rsvd_dek_ptr;
 	uint8_t rsvd2[4];
 	uint8_t keytag[8];
