@@ -42,7 +42,6 @@
 #include <assert.h>
 #include <stdarg.h>
 #include <stddef.h>
-#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -133,7 +132,7 @@ static int check_process_uniqueness(struct config_t *conf)
 	char path[256];
 	int fd;
 
-	snprintf(path, sizeof(path), SRP_DEAMON_LOCK_PREFIX "_%s_%d",
+	snprintf(path, sizeof(path), SRP_DAEMON_LOCK_PREFIX "_%s_%d",
 		 conf->dev_name, conf->port_num);
 
 	if ((fd = open(path, O_CREAT|O_RDWR,
@@ -142,7 +141,6 @@ static int check_process_uniqueness(struct config_t *conf)
 		return -1;
 	}
 
-	fchmod(fd, S_IRUSR|S_IRGRP|S_IROTH|S_IWUSR|S_IWGRP|S_IWOTH);
 	if (0 != lockf(fd, F_TLOCK, 0)) {
 		pr_err("failed to lock %s (errno: %d). possibly another "
 		       "srp_daemon is locking it\n", path, errno);
@@ -231,7 +229,7 @@ static void usage(const char *argv0)
 	fprintf(stderr, "-R <rescan time>	perform complete Rescan every <rescan time> seconds\n");
 	fprintf(stderr, "-T <retry timeout>	Retries to connect to existing target after Timeout of <retry timeout> seconds\n");
 	fprintf(stderr, "-l <tl_retry timeout>	Transport retry count before failing IO. should be in range [2..7], (default 2)\n");
-	fprintf(stderr, "-f <rules file>	use rules File to set to which target(s) to connect (default: " SRP_DEAMON_CONFIG_FILE ")\n");
+	fprintf(stderr, "-f <rules file>	use rules File to set to which target(s) to connect (default: " SRP_DAEMON_CONFIG_FILE ")\n");
 	fprintf(stderr, "-t <timeout>		Timeout for mad response in milliseconds\n");
 	fprintf(stderr, "-r <retries>		number of send Retries for each mad\n");
 	fprintf(stderr, "-n 			New connection command format - use also initiator extension\n");
@@ -1505,6 +1503,7 @@ static int parse_other_option(struct rule *rule, char *ptr)
 {
 	static const char *const opt[] = {
 		"allow_ext_sg=",
+		"ch_count=",
 		"cmd_sg_entries=",
 		"comp_vector=",
 		"max_cmd_per_lun=",
@@ -1735,7 +1734,7 @@ static int get_config(struct config_t *conf, int argc, char *argv[])
 	conf->retry_timeout 		= 20;
 	conf->add_target_file  		= NULL;
 	conf->print_initiator_ext	= 0;
-	conf->rules_file		= SRP_DEAMON_CONFIG_FILE;
+	conf->rules_file		= SRP_DAEMON_CONFIG_FILE;
 	conf->rules			= NULL;
 	conf->tl_retry_count		= 0;
 
@@ -1884,7 +1883,6 @@ static void umad_resources_init(struct umad_resources *umad_res)
 {
 	umad_res->portid = -1;
 	umad_res->agent = -1;
-	umad_res->agent = -1;
 	umad_res->port_sysfs_path = NULL;
 }
 
@@ -1902,6 +1900,27 @@ static void umad_resources_destroy(struct umad_resources *umad_res)
 	umad_done();
 }
 
+static int check_link_layer(const char *port_sysfs_path)
+{
+	const char expected_link_layer[] = "InfiniBand";
+	char link_layer[sizeof(expected_link_layer)];
+	int ret;
+
+	ret = srpd_sys_read_string(port_sysfs_path, "link_layer", link_layer,
+				   sizeof(link_layer));
+	if (ret < 0) {
+		pr_err("Couldn't read link layer\n");
+		return ret;
+	}
+
+	if (strcmp(link_layer, expected_link_layer)) {
+		pr_err("Unsupported link layer %s\n", link_layer);
+		return -EINVAL;
+	}
+
+	return 0;
+}
+
 static int umad_resources_create(struct umad_resources *umad_res)
 {
 
@@ -1915,6 +1934,10 @@ static int umad_resources_create(struct umad_resources *umad_res)
 		return -ENOMEM;
 	}
 
+	ret = check_link_layer(umad_res->port_sysfs_path);
+	if (ret)
+		return ret;
+
 	umad_res->portid = umad_open_port(config->dev_name, config->port_num);
 	if (umad_res->portid < 0) {
 		pr_err("umad_open_port failed for device %s port %d\n",
