@@ -9,7 +9,7 @@ import errno
 
 from pyverbs.pyverbs_error import PyverbsRDMAError
 from pyverbs.providers.mlx5.mlx5dv import Mlx5UAR
-import pyverbs.providers.mlx5.mlx5_enums as e
+from pyverbs.providers.mlx5.mlx5_enums import _MLX5DV_UAR_ALLOC_TYPE_BF, _MLX5DV_UAR_ALLOC_TYPE_NC
 from tests.mlx5_base import Mlx5RDMATestCase
 from tests.base import BaseResources
 
@@ -27,7 +27,7 @@ class Mlx5UarTestCase(Mlx5RDMATestCase):
 
     def test_alloc_uar(self):
         try:
-            for f in [e._MLX5DV_UAR_ALLOC_TYPE_BF, e._MLX5DV_UAR_ALLOC_TYPE_NC]:
+            for f in [_MLX5DV_UAR_ALLOC_TYPE_BF, _MLX5DV_UAR_ALLOC_TYPE_NC]:
                 self.uar_res.uars.append(Mlx5UAR(self.uar_res.ctx, f))
         except PyverbsRDMAError as ex:
             if ex.error_code == errno.EOPNOTSUPP or ex.error_code == errno.EPROTONOSUPPORT:
